@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1afmtool.c 
   ----- Author:      Rainer Menzner (Rainer.Menzner@web.de)
-  ----- Date:        2005-05-01
+  ----- Date:        2007-12-23
   ----- Description: This file is part of the t1-library. It contains
                      functions for generating a fallback set of afm data
 		     from type 1 font files.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2005.
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2007.
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -90,6 +90,13 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
   char **charnames;
   int nochars=0;
   FontInfo *pAFMData;
+
+  /* When generaing fallback info, we accumulate a font bounding box that
+     could be useful when the font's definition is missing or trivial. */
+  int acc_llx=0;
+  int acc_lly=0;
+  int acc_urx=0;
+  int acc_ury=0;
   
   
   /* We return to this if something goes wrong deep in the rasterizer */
@@ -237,6 +244,21 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
 	pAFMData->cmi[i].charBBox.ury =0;
       }
       pAFMData->cmi[i].ligs=NULL;
+
+      /* Accumulate bounding box of font */
+      if ( pAFMData->cmi[i].charBBox.llx < acc_llx ) {
+	acc_llx=pAFMData->cmi[i].charBBox.llx;
+      }
+      if ( pAFMData->cmi[i].charBBox.lly < acc_lly ) {
+	acc_lly=pAFMData->cmi[i].charBBox.lly;
+      }
+      if ( pAFMData->cmi[i].charBBox.urx > acc_urx ) {
+	acc_urx=pAFMData->cmi[i].charBBox.urx;
+      }
+      if ( pAFMData->cmi[i].charBBox.ury > acc_ury ) {
+	acc_ury=pAFMData->cmi[i].charBBox.ury;
+      }
+      
       /* We are done with area, so get rid of it. Solves the REALLY
 	 HUGE memory leak */
       KillRegion (area);
@@ -247,6 +269,24 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
 	   nochars, FontID);
   T1_PrintLog( "T1_GenerateAFMFallbackInfo()", err_warn_msg_buf,
 	       T1LOG_STATISTIC);
+
+  /* Check whether the bounding box we computed could be better than that
+     specified in the font file itself. Id so, we overwrite it. */
+  if ( pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[0].data.integer == 0   &&
+       pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[1].data.integer == 0   &&
+       pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[2].data.integer == 0   &&
+       pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[3].data.integer == 0 ) {
+    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[0].data.integer = acc_llx;  
+    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[1].data.integer = acc_lly;  
+    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[2].data.integer = acc_urx;  
+    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTBBOX].value.data.arrayP[3].data.integer = acc_ury;
+    
+    sprintf( err_warn_msg_buf,
+	     "Substituted accumulated FontBBox [%d,%d,%d,%d] for trivial FontBBox of font %d!",
+	     acc_llx, acc_lly, acc_urx, acc_ury, FontID);
+    T1_PrintLog( "T1_GenerateAFMFallbackInfo()", err_warn_msg_buf,
+		 T1LOG_WARNING);
+  }
   
   /* make sure to free S */
   if (S) {
