@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1env.c 
   ----- Author:      Rainer Menzner (Rainer.Menzner@web.de)
-  ----- Date:        2005-05-17
+  ----- Date:        2007-12-22
   ----- Description: This file is part of the t1-library. It implements
                      the reading of a configuration file and path-searching
 		     of type1-, afm- and encoding files.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2005.
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2007.
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -601,7 +601,7 @@ char *intT1_Env_GetCompletePath( char *FileName,
     /* cut a trailing directory separator */
     j=strlen(pathbuf);
     if (pathbuf[j-1]==DIRECTORY_SEP_CHAR)
-      pathbuf[j--]='\0';
+      pathbuf[--j]='\0';
     /* Add the directory separator: */
 #ifdef VMS
     { char *p= strrchr(pathbuf, DIRECTORY_SEP_CHAR);
@@ -610,7 +610,24 @@ char *intT1_Env_GetCompletePath( char *FileName,
     } 
 #endif 
     strcat( pathbuf, DIRECTORY_SEP);
-    /* And finally the filename: */
+    /* And finally the filename.
+       The following is fix against a vulnerability given by passing in
+       large filenames, cf.:
+
+           http://www.securityfocus.com/bid/25079
+
+       or
+
+           http://packetstormsecurity.nl/0707-advisories/t1lib.txt
+
+       If current pathbuf + StrippedName + 1 byte for NULL is bigger than
+       pathbuf log a warning and try next pathbuf */
+    if ( strlen(pathbuf) + strlen(StrippedName) + 1 > sizeof(pathbuf) ) {
+      T1_PrintLog( "intT1_Env_GetCompletePath()", "Omitting suspicious long candidate path in order to prevent buffer overflow.",
+		   T1LOG_WARNING);
+      i++;
+      continue;
+    }
     strcat( pathbuf, StrippedName);
     
     /* Check for existence of the path: */
